﻿using System;
using Telerik.Windows.Controls;
using Telerik.Windows.Documents.Model;

namespace OutlookTemplate
{
    public class SelectedEmailViewModel : ViewModelBase
    {
        private bool hasEmail;
        private Email email;
        private RadDocument originalContent;
        private string subject;
        private bool isInEditMode;
        private string to;

        public SelectedEmailViewModel()
        {
            this.ReplyCommand = new DelegateCommand(this.OnReplyCommandExecute, o => this.HasEmail);
            this.ReplyAllCommand = new DelegateCommand(this.OnReplyAllCommandExecute, o => this.HasEmail);
            this.ForwardCommand = new DelegateCommand(this.OnForwardCommandExecute, o => this.HasEmail);
            this.DiscardCommand = new DelegateCommand(this.OnDiscardCommandExecute, o => this.HasEmail);
        }

        public DelegateCommand ReplyCommand { get; private set; }
        public DelegateCommand ReplyAllCommand { get; private set; }
        public DelegateCommand ForwardCommand { get; private set; }
        public DelegateCommand DiscardCommand { get; private set; }

        public RadDocument OriginalContent
        {
            get
            {
                return this.originalContent;
            }
            set
            {
                if (this.originalContent != value)
                {
                    this.originalContent = value;
                    this.OnPropertyChanged(() => this.OriginalContent);
                }
            }
        }

        /// <summary>
        /// Gets or sets To and notifies for changes
        /// </summary>
        public string To
        {
            get
            {
                return this.to;
            }

            set
            {
                if (this.to != value)
                {
                    this.to = value;
                    this.OnPropertyChanged(() => this.To);
                }
            }
        }

        /// <summary>
        /// Gets or sets Subject and notifies for changes
        /// </summary>
        public string Subject
        {
            get
            {
                return this.subject;
            }

            set
            {
                if (this.subject != value)
                {
                    this.subject = value;
                    this.OnPropertyChanged(() => this.Subject);
                }
            }
        }

        /// <summary>
        /// Gets or sets IsInEditMode and notifies for changes
        /// </summary>
        public bool IsInEditMode
        {
            get
            {
                return this.isInEditMode;
            }

            set
            {
                if (this.isInEditMode != value)
                {
                    this.isInEditMode = value;
                    this.OnPropertyChanged(() => this.IsInEditMode);
                }
            }
        }

        /// <summary>
        /// Gets or sets HasEmail and notifies for changes
        /// </summary>
		public bool HasEmail
        {
            get
            {
                return this.hasEmail;
            }
            set
            {
                if (this.hasEmail != value)
                {
                    this.hasEmail = value;
                    this.ReplyCommand.InvalidateCanExecute();
                    this.ReplyAllCommand.InvalidateCanExecute();
                    this.ForwardCommand.InvalidateCanExecute();
                    this.DiscardCommand.InvalidateCanExecute();
                    this.OnPropertyChanged(() => this.HasEmail);
                }
            }
        }

        /// <summary>
        /// Gets or sets Email and notifies for changes
        /// </summary>
		public Email Email
        {
            get
            {
                return this.email;
            }
            set
            {
                if (this.email != value)
                {
                    this.email = value;
                    if (value != null && this.email.Sender != null)
                    {
                        this.OriginalContent = (RadDocument)this.email.Content.CreateDeepCopy();
                        this.HasEmail = true;
                    }
                    else
                    {
                        this.HasEmail = false;
                        this.IsInEditMode = false;
                    }

                    this.OnPropertyChanged(() => this.Email);
                }
            }
        }

        private void OnReplyCommandExecute(object obj)
        {
            this.To = this.Email.Sender;
            this.Subject = String.Format("Re: {0}", this.Email.Subject);
            this.IsInEditMode = true;
        }

        private void OnDiscardCommandExecute(object obj)
        {
            this.IsInEditMode = false;
        }

        private void OnReplyAllCommandExecute(object obj)
        {
            this.To = String.Format("{0};{1}", this.Email.Sender, this.Email.Recipient);
            this.Subject = String.Format("RE: {0}", this.Email.Subject);
            this.IsInEditMode = true;
        }

        private void OnForwardCommandExecute(object obj)
        {
            this.To = this.Email.Sender;
            this.Subject = String.Format("FW: {0}", this.Email.Subject);
            this.IsInEditMode = true;
        }
    }
}