﻿using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Controls;
using Telerik.Windows.Data;

namespace OutlookTemplate
{
    /// <summary>
    /// Interaction logic for MainView.xaml
    /// </summary>
    public partial class MainView : UserControl
    {
        private MainViewModel mainViewModel;
        private FilterDescriptor unreadDescriptor;

        public MainView()
        {
            InitializeComponent();
            this.InitializeDataContext();
        }

        private void InitializeDataContext()
        {
            this.mainViewModel = new MainViewModel();
            this.DataContext = this.mainViewModel;
            this.mainViewModel.PropertyChanged += this.OnMainViewPropertyChanged;
            this.unreadDescriptor = new FilterDescriptor
            {
                Member = "Status",
                Operator = FilterOperator.IsEqualTo,
                Value = "Unread",
            };
        }

        private void OnMainViewPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "SelectedEmail" && this.mainViewModel.SelectedEmailViewModel.Email != null)
            {
                this.nonEditableRichTextBox.Document = this.mainViewModel.SelectedEmailViewModel.Email.Content;
                this.editableRichTextBox.Document = this.mainViewModel.SelectedEmailViewModel.OriginalContent;
            }
        }

        private void OnAllRadioButtonClick(object sender, RoutedEventArgs e)
        {
            this.gridView.FilterDescriptors.Remove(this.unreadDescriptor);
        }

        private void OnUnreadRadioButtonClick(object sender, RoutedEventArgs e)
        {
            this.gridView.FilterDescriptors.Add(this.unreadDescriptor);
        }

        private void OnGridViewSelectionChanged(object sender, SelectionChangeEventArgs e)
        {
            if (this.mainViewModel.SelectedEmail != null)
            {
                this.mainViewModel.SelectedEmail.Status = Enums.EmailStatus.Read;
            }
        }

        private void OnNewMailClicked(object sender, RoutedEventArgs e)
        {
            var window = new NewEmailWindow { DataContext = this.DataContext };
            window.Owner = Application.Current.MainWindow;
            window.Show();
        }

        private void OnPopOutClicked(object sender, RoutedEventArgs e)
        {
            var newViewModel = this.mainViewModel;
            newViewModel.SelectedEmailViewModel.Email.Content = this.mainViewModel.SelectedEmailViewModel.OriginalContent;
            var window = new NewEmailWindow(newViewModel.SelectedEmailViewModel);
            window.Owner = Application.Current.MainWindow;
            window.Show();
            this.mainViewModel.SelectedEmailViewModel.IsInEditMode = false;
        }
    }
}